//  ========================================================================
/**
 * @copyright Copyright (C) 1999 Technical Pursuit Inc. (TPI) All Rights
 *     Reserved. Patents Pending, Technical Pursuit Inc. Licensed under the
 *     OSI-approved Reciprocal Public License (RPL) Version 1.5. See the RPL
 *     for your rights and responsibilities. Contact TPI to purchase optional
 *     privacy waivers if you must keep your TIBET-based source code private.
 * @overview The 'tibet quickstart' command. A simple command whose only goal
 *     is to output help on the steps to take after installation to get a new
 *     project up an running with a minimum of overhead.
 */
//  ========================================================================

/* eslint indent:0 */

(function() {

'use strict';

var CLI,
    Cmd;

CLI = require('./_cli');

//  ---
//  Type Construction
//  ---

Cmd = function() { /* init */ };
Cmd.Parent = require('./_cmd');
Cmd.prototype = new Cmd.Parent();


//  ---
//  Type Attributes
//  ---

/**
 * The command execution context.
 * @type {Cmd.CONTEXTS}
 */
Cmd.CONTEXT = CLI.CONTEXTS.ANY;

/**
 * The command name for this type.
 * @type {string}
 */
Cmd.NAME = 'quickstart';

//  ---
//  Instance Attributes
//  ---

/**
 * The command usage string.
 * @type {string}
 */
Cmd.prototype.USAGE = 'tibet quickstart';


//  ---
//  Instance Methods
//  ---

/**
 * Perform the actual command processing logic.
 * @returns {Number|Promise} The return code produced by running the command (a
 *     non-zero indicates an Error) or a Promise that resolves when the command
 *     finishes.
 */
Cmd.prototype.execute = function() {

    var str;

    str = `
Welcome to TIBET!

The 'tibet clone' command is your first step in creating a TIBET project.

Before using 'clone', navigate to a directory to hold your project content
and select a name for your new project. The name will be used as a directory
name so it should be a valid directory name on your platform.

Type 'tibet clone {appname}', replacing {appname} with your project name:

    $ tibet clone hello
    ...
    Application DNA 'electron' cloned to ./hello as 'hello'.
    cd ./hello; tibet init; to continue developing.

With your new project in place you need to initialize it to install any code
dependencies specific to the template you cloned.

Navigate to your project and then type 'tibet init' to initialize it:

    $ cd hello
    $ tibet init
    ...
    project initialized successfully.

All project DNA templates support using 'tibet start' to run them.
Electron projects will launch directly. TIBET Data Server (TDS) projects will
start their Express-based NodeJS server (using port 1407 by default):

    $ tibet start
    ...

Congratulations! Your new TIBET project is running.

For Electron projects your application is now running and ready for expansion.
For TDS-based projects open the web address output by 'tibet start' for your
project in a supported HTML5 browser. All TIBET projects display launch text
directing you on how to take the next step in your TIBET journey.

For more info visit https://gettibet.app
`;

    this.info(str);

    return 0;
};


module.exports = Cmd;

}());
