//  ========================================================================
/**
 * @copyright Copyright (C) 1999 Technical Pursuit Inc. (TPI) All Rights
 *     Reserved. Patents Pending, Technical Pursuit Inc. Licensed under the
 *     OSI-approved Reciprocal Public License (RPL) Version 1.5. See the RPL
 *     for your rights and responsibilities. Contact TPI to purchase optional
 *     privacy waivers if you must keep your TIBET-based source code private.
 */
//  ------------------------------------------------------------------------

/**
 * @type {TP.tsh.keys}
 * @summary Dumps a list of keyboard shortcuts for the Sherpa.
 */

//  ------------------------------------------------------------------------

TP.tag.ActionTag.defineSubtype('tsh:keys');

TP.tsh.keys.addTraitTypes(TP.tsh.Element);

//  ------------------------------------------------------------------------
//  Type Methods
//  ------------------------------------------------------------------------

TP.tsh.keys.Type.defineMethod('tshExecute',
async function(aRequest) {

    /**
     * @method tshExecute
     * @summary Runs the receiver, effectively invoking its action.
     * @param {TP.sig.Request} aRequest The request containing command input for
     *     the shell.
     * @returns {TP.sig.Request|Number} The request or a TSH shell loop control
     *     constant which controls how the outer TSH processing loop should
     *     continue. Common values are TP.CONTINUE, TP.DESCEND, and TP.BREAK.
     */

    var shell,
        keys,

        output;

    shell = aRequest.at('cmdShell');

    if (await shell.getArgument(aRequest, 'tsh:debug', null, false)) {
        return this.printDebug(aRequest, true, false);
    }

    if (await shell.getArgument(aRequest, 'tsh:debugresolve', null, false)) {
        return this.printDebug(aRequest, true, true);
    }

    keys = TP.hc(
        'Alt-ArrowUp', 'Show/Hide HUD',
        'Shift-Shift', 'Focus Input Cell',
        'Shift-Return', 'Execute',

        'Ctrl-U', 'Clears the input cell of content',
        'Ctrl-K', 'Clears the output area of all output cells',
        'Ctrl-A', 'Activates auto-complete mode.',

        'Ctrl-ArrowDown', 'Cycle output mode downward (will wrap)',
        'Ctrl-ArrowUp', 'Cycle output mode upward (will wrap)',

        'Shift-ArrowDown', 'Move to "next" history entry (will not wrap)',
        'Shift-ArrowUp', 'Move to "last" history entry (will not wrap)',

        'ArrowDown', 'Scroll last output cell down 1 line',
        'ArrowUp', 'Scroll last output cell up 1 line',

        'PageDown', 'Scroll last output cell down 1 page',
        'PageUp', 'Scroll last output cell up 1 page',

        'Escape', 'Cancels an autocomplete operation',
        'Shift-Escape', 'Cancels a multi-prompt request sequence'
    );

    output = '<dl>';

    keys.perform(
        function(kvPair) {
            output += '<dt>' + kvPair.first() + '</dt>' +
                        '<dd>' + kvPair.last() + '</dd>';
        });

    output += '</dl>';

    aRequest.atPut('cmdAsIs', true);

    return aRequest.complete(output);
});

//  ------------------------------------------------------------------------

TP.shell.TSH.addHelpTopic('keys',
    TP.tsh.keys.Type.getMethod('tshExecute'),
    'Lists keyboard shortcuts for the Sherpa toolset.',
    ':keys',
    'Coming Soon');

//  ------------------------------------------------------------------------
//  end
//  ========================================================================
