//  ========================================================================
/**
 * @copyright Copyright (C) 1999 Technical Pursuit Inc. (TPI) All Rights
 *     Reserved. Patents Pending, Technical Pursuit Inc. Licensed under the
 *     OSI-approved Reciprocal Public License (RPL) Version 1.5. See the RPL
 *     for your rights and responsibilities. Contact TPI to purchase optional
 *     privacy waivers if you must keep your TIBET-based source code private.
 */
//  ========================================================================

/*
 * Tests for the TP.core.AsyncQueue type.
 */

//  ------------------------------------------------------------------------

TP.core.AsyncQueue.Type.describe('queueing',
function() {

    this.it('3 delayed Promises - no queueing', async function(test, options) {
        var promises,

            promise1End,
            promise2End,
            promise3End;

        promises = TP.ac();

        promises.push(
            Promise.delay('promise1', 100).then(
                function() {
                    promise1End = Date.now();
                }),
            Promise.delay('promise2', 500).then(
                function() {
                    promise2End = Date.now();
                }),
            Promise.delay('promise3', 300).then(
                function() {
                    promise3End = Date.now();
                })
        );

        await Promise.all(promises);

        //  Promise.all does not guarantee order. Since Promise2 finishes later
        //  than Promise3, the order is Promise1, Promise3, Promise2
        test.assert.isTrue(promise1End < promise3End);
        test.assert.isTrue(promise3End < promise2End);
        test.assert.isTrue(promise1End < promise2End);
    });

    this.it('3 delayed Promises - queueing', async function(test, options) {
        var promiseQueue,

            promise1End,
            promise2End,
            promise3End;

        promiseQueue = TP.core.AsyncQueue.construct();

        promiseQueue.enqueue(
            function() {
                return Promise.delay('promise1', 100).then(
                    function() {
                        promise1End = Date.now();
                    });
            });

        promiseQueue.enqueue(
            function() {
                return Promise.delay('promise2', 500).then(
                    function() {
                        promise2End = Date.now();
                    });
            });

        promiseQueue.enqueue(
            function() {
                return Promise.delay('promise3', 300).then(
                    function() {
                        promise3End = Date.now();
                    });
            });

        await test.andWait(2000);

        //  Our async queue does guarantee order. It doesn't matter that
        //  Promise2 finishes later than Promise3... here the order is the order
        //  in which they were queued: Promise1, Promise2, Promise3
        test.assert.isTrue(promise1End < promise3End);
        test.assert.isTrue(promise1End < promise2End);
        test.assert.isTrue(promise2End < promise3End);
    });

});

//  ------------------------------------------------------------------------
//  end
//  ========================================================================
