//  ========================================================================
//  xctrls:button
//  ========================================================================

TP.xctrls.button.Type.describe('TP.xctrls.button: manipulation',
function() {

    var driver,
        windowContext,

        unloadURI,
        loadURI;

    unloadURI = TP.uc(TP.sys.cfg('path.blank_page'));

    //  ---

    this.before(
        async function(suite, options) {

            driver = this.getDriver();
            windowContext = driver.get('windowContext');

            loadURI = TP.uc('~lib_test/src/xctrls/xctrls_button.xhtml');
            await driver.setLocation(loadURI);

            this.startTrackingSignals();
        });

    //  ---

    this.after(
        async function(suite, options) {

            this.stopTrackingSignals();

            //  Unload the current page by setting it to the blank
            await driver.setLocation(unloadURI);

            //  Unregister the URI to avoid a memory leak
            loadURI.unregister();
        });

    //  ---

    this.afterEach(
        function(test, options) {
            this.getSuite().resetSignalTracking();
        });

    //  ---

    this.it('Focusing', async function(test, options) {

        var button,
            focusedElem;

        button = TP.byId('button1', windowContext);

        //  Change the focus via 'direct' method

        await driver.constructSequence().
                        sendEvent(TP.hc('type', 'focus'), button).
                        run();

        test.assert.hasAttribute(button, 'pclass:focus');

        await test.assert.didSignal(button, 'TP.sig.UIFocus');
        await test.assert.didSignal(button, 'TP.sig.UIDidFocus');

        focusedElem = driver.getFocusedElement();
        test.assert.isIdenticalTo(focusedElem, button);
    });

    //  ---

    this.it('Activation - mouse', async function(test, options) {

        var button;

        //  Change the content via 'user' interaction

        button = TP.byId('button1', windowContext);

        //  Individual mousedown/mouseup

        await driver.constructSequence().
                        mouseDown(TP.LEFT, button).
                        run();

        test.assert.hasAttribute(button, 'pclass:active');

        await test.assert.didSignal(button, 'TP.sig.UIActivate');
        await test.assert.didSignal(button, 'TP.sig.UIDidActivate');

        test.getSuite().resetSignalTracking();

        await driver.constructSequence().
                        mouseUp(TP.LEFT, button).
                        run();

        test.refute.hasAttribute(button, 'pclass:active');

        await test.assert.didSignal(button, 'TP.sig.UIDeactivate');
        await test.assert.didSignal(button, 'TP.sig.UIDidDeactivate');

        test.getSuite().resetSignalTracking();

        //  click

        await driver.constructSequence().
                        click(TP.LEFT, button).
                        run();

        //  Don't test the attribute here - it will already have been
        //  removed.

        await test.assert.didSignal(button, 'TP.sig.UIActivate');
        await test.assert.didSignal(button, 'TP.sig.UIDidActivate');

        await test.assert.didSignal(button, 'TP.sig.UIDeactivate');
        await test.assert.didSignal(button, 'TP.sig.UIDidDeactivate');
    });

    //  ---

    this.it('Activation - keyboard', async function(test, options) {

        var button;

        //  Change the content via 'user' interaction

        button = TP.byId('button1', windowContext);

        //  Individual keydown/keyup

        await driver.constructSequence().
                        keyDown('[Enter]', button).
                        run();

        test.assert.hasAttribute(button, 'pclass:active');

        await test.assert.didSignal(button, 'TP.sig.UIActivate');
        await test.assert.didSignal(button, 'TP.sig.UIDidActivate');

        test.getSuite().resetSignalTracking();

        await driver.constructSequence().
                        keyUp('[Enter]', button).
                        run();

        test.refute.hasAttribute(button, 'pclass:active');

        await test.assert.didSignal(button, 'TP.sig.UIDeactivate');
        await test.assert.didSignal(button, 'TP.sig.UIDidDeactivate');
    });

    //  ---

    this.it('Disabled behavior', async function(test, options) {

        var button;

        //  All of these mechanisms to 'activate' the control should fail - it's
        //  disabled.

        //  Disable it
        button = TP.byId('button1', windowContext);
        await button.setAttrDisabled(true);

        //  --- Focus

        await driver.constructSequence().
                        sendEvent(TP.hc('type', 'focus'), button).
                        run();

        test.refute.hasAttribute(button, 'pclass:focus');

        await test.refute.didSignal(button, 'TP.sig.UIFocus');
        await test.refute.didSignal(button, 'TP.sig.UIDidFocus');

        test.getSuite().resetSignalTracking();

        //  --- Individual mousedown/mouseup

        await driver.constructSequence().
                        mouseDown(TP.LEFT, button).
                        run();

        test.refute.hasAttribute(button, 'pclass:active');

        await test.refute.didSignal(button, 'TP.sig.UIActivate');
        await test.refute.didSignal(button, 'TP.sig.UIDidActivate');

        await driver.constructSequence().
                        mouseUp(TP.LEFT, button).
                        run();

        await test.refute.didSignal(button, 'TP.sig.UIDeactivate');
        await test.refute.didSignal(button, 'TP.sig.UIDidDeactivate');

        test.getSuite().resetSignalTracking();

        //  --- click

        await driver.constructSequence().
                        click(TP.LEFT, button).
                        run();

        await test.refute.didSignal(button, 'TP.sig.UIActivate');
        await test.refute.didSignal(button, 'TP.sig.UIDidActivate');

        await test.refute.didSignal(button, 'TP.sig.UIDeactivate');
        await test.refute.didSignal(button, 'TP.sig.UIDidDeactivate');

        test.getSuite().resetSignalTracking();

        //  --- Individual keydown/keyup

        await driver.constructSequence().
                        keyDown('[Enter]', button).
                        run();

        test.refute.hasAttribute(button, 'pclass:active');

        await test.refute.didSignal(button, 'TP.sig.UIActivate');
        await test.refute.didSignal(button, 'TP.sig.UIDidActivate');

        test.getSuite().resetSignalTracking();

        await driver.constructSequence().
                        keyUp('[Enter]', button).
                        run();

        await test.refute.didSignal(button, 'TP.sig.UIDeactivate');
        await test.refute.didSignal(button, 'TP.sig.UIDidDeactivate');
    });
});

//  ------------------------------------------------------------------------
//  end
//  ========================================================================
