//  ========================================================================
//  TP.tmp.PopupTestContent
//  ========================================================================

/**
 * @type {TP.tmp.PopupTestContent}
 * @summary TP.tmp.PopupTestContent
 */

//  ------------------------------------------------------------------------

TP.tag.ComputedTag.defineSubtype('tmp.PopupTestContent');

//  ------------------------------------------------------------------------
//  Type Attributes
//  ------------------------------------------------------------------------

//  This tag has no associated CSS. Note how these properties are TYPE_LOCAL, by
//  design.
TP.tmp.PopupTestContent.defineAttribute('styleURI', TP.NO_RESULT);
TP.tmp.PopupTestContent.defineAttribute('themeURI', TP.NO_RESULT);

//  ------------------------------------------------------------------------
//  Type Methods
//  ------------------------------------------------------------------------

TP.tmp.PopupTestContent.Type.defineMethod('tagCompile',
function(aRequest) {

    /**
     * @method tagCompile
     * @summary Convert the receiver into a format suitable for inclusion in a
     *     markup DOM.
     * @param {TP.sig.ShellRequest} aRequest The request containing command
     *     input for the shell.
     * @returns {Element} The new element.
     */

    var elem,
        newElem;

    //  Make sure that we have an element to work from.
    if (!TP.isElement(elem = aRequest.at('node'))) {
        return;
    }

    newElem = TP.xhtmlnode(
    '<div tibet:tag="tmp:PopupTestContent"' +
            ' class="type_test_content">' +
        '<ul>' +
            '<li>' + 'Item #1' + '</li>' +
            '<li>' + 'Item #2' + '</li>' +
            '<li>' + 'Item #3' + '</li>' +
            '<li>' + 'Item #4' + '</li>' +
            '<li>' + 'Item #5' + '</li>' +
        '</ul>' +
    '</div>');

    //  Note here how we return the *result* of this method due to node
    //  importing, etc.
    return TP.elementReplaceWith(elem, newElem);
});

//  ========================================================================
//  xctrls:popup
//  ========================================================================

TP.xctrls.popup.Type.describe('TP.xctrls.popup: manipulation',
function() {

    var driver,
        windowContext,

        unloadURI,
        loadURI;

    unloadURI = TP.uc(TP.sys.cfg('path.blank_page'));

    //  ---

    this.before(
        async function(suite, options) {

            driver = this.getDriver();
            windowContext = driver.get('windowContext');

            loadURI = TP.uc('~lib_test/src/xctrls/xctrls_popup.xhtml');
            await driver.setLocation(loadURI);

            this.startTrackingSignals();
        });

    //  ---

    this.after(
        async function(suite, options) {

            this.stopTrackingSignals();

            //  Unload the current page by setting it to the blank
            await driver.setLocation(unloadURI);

            //  Unregister the URI to avoid a memory leak
            loadURI.unregister();
        });

    //  ---

    this.afterEach(
        function(test, options) {
            this.getSuite().resetSignalTracking();
        });

    //  ---

    this.it('Focusing', async function(test, options) {

        var trigger,
            focusedElem;

        trigger = TP.byId('trigger3', windowContext);

        //  Change the focus via 'direct' method

        await driver.constructSequence().
                        sendEvent(TP.hc('type', 'focus'), trigger).
                        run();

        test.assert.hasAttribute(trigger, 'pclass:focus');

        await test.assert.didSignal(trigger, 'TP.sig.UIFocus');
        await test.assert.didSignal(trigger, 'TP.sig.UIDidFocus');

        focusedElem = driver.getFocusedElement();
        test.assert.isIdenticalTo(focusedElem, trigger);
    });

    //  ---

    this.it('Activation - mouse - non-sticky', async function(test, options) {

        var trigger,
            systemPopup;

        //  Change the content via 'user' interaction

        trigger = TP.byId('trigger3', windowContext);

        //  TP.xctrls.popup doesn't respond to individual mousedown/mouseup. It
        //  is triggered by another control wanting to show/hide the popup.

        //  It will, however, respond to click to show/hide and will hide
        //  immediately because it's not sticky - trigger3 is not sticky.

        await driver.constructSequence().
                        click(TP.LEFT, trigger).
                        run();

        //  Don't test the attribute here - it will already have been
        //  removed.

        await test.assert.didSignal(trigger, 'TP.sig.UIActivate');
        await test.assert.didSignal(trigger, 'TP.sig.UIDidActivate');

        await test.assert.didSignal(trigger, 'TP.sig.UIDeactivate');
        await test.assert.didSignal(trigger, 'TP.sig.UIDidDeactivate');

        systemPopup = TP.byId('systemPopup', windowContext);
        test.refute.isVisible(systemPopup);
    });

    //  ---

    this.it('Activation - keyboard - non-sticky', async function(test, options) {

        var trigger,
            systemPopup;

        //  Change the content via 'user' interaction

        trigger = TP.byId('trigger3', windowContext);

        //  Individual keydown/keyup

        //  TP.xctrls.popup will respond to keydown/keyup to show/hide, and will
        //  hide immediately because it's not sticky - trigger3 is not sticky.

        await driver.constructSequence().
                        keyDown('[Enter]', trigger).
                        run();

        test.assert.hasAttribute(trigger, 'pclass:active');

        await test.assert.didSignal(trigger, 'TP.sig.UIActivate');
        await test.assert.didSignal(trigger, 'TP.sig.UIDidActivate');

        systemPopup = TP.byId('systemPopup', windowContext);
        test.assert.isVisible(systemPopup);

        test.getSuite().resetSignalTracking();

        await driver.constructSequence().
                        keyUp('[Enter]', trigger).
                        run();

        test.refute.hasAttribute(trigger, 'pclass:active');

        await test.assert.didSignal(trigger, 'TP.sig.UIDeactivate');
        await test.assert.didSignal(trigger, 'TP.sig.UIDidDeactivate');

        systemPopup = TP.byId('systemPopup', windowContext);
        test.refute.isVisible(systemPopup);
    });

    //  ---

    this.it('Activation - mouse - sticky', async function(test, options) {

        var trigger,
            systemPopup;

        //  Change the content via 'user' interaction

        trigger = TP.byId('trigger1', windowContext);

        //  TP.xctrls.popup doesn't respond to individual mousedown/mouseup. It
        //  is triggered by another control wanting to show/hide the popup.

        //  It will, however, respond to click to show/hide and will not hide if
        //  it's 'sticky' - trigger1 is sticky.

        await driver.constructSequence().
                        click(TP.LEFT, trigger).
                        run();

        //  Don't test the attribute here - it will already have been
        //  removed.

        await test.assert.didSignal(trigger, 'TP.sig.UIActivate');
        await test.assert.didSignal(trigger, 'TP.sig.UIDidActivate');

        systemPopup = TP.byId('systemPopup', windowContext);
        test.assert.isVisible(systemPopup);

        test.getSuite().resetSignalTracking();

        await driver.constructSequence().
                        click(TP.LEFT, trigger).
                        run();

        test.refute.hasAttribute(trigger, 'pclass:active');

        await test.assert.didSignal(trigger, 'TP.sig.UIDeactivate');
        await test.assert.didSignal(trigger, 'TP.sig.UIDidDeactivate');

        systemPopup = TP.byId('systemPopup', windowContext);
        test.refute.isVisible(systemPopup);
    });

    //  ---

    this.it('Activation - keyboard - sticky', async function(test, options) {

        var trigger,
            systemPopup;

        //  Change the content via 'user' interaction

        trigger = TP.byId('trigger1', windowContext);

        //  Individual keydown/keyup

        //  TP.xctrls.popup will respond to keydown/keyup to show/hide, and will
        //  not hide if it's not sticky - trigger1 is sticky.

        await driver.constructSequence().
                        keyDown('[Enter]', trigger).
                        run();

        test.assert.hasAttribute(trigger, 'pclass:active');

        await test.assert.didSignal(trigger, 'TP.sig.UIActivate');
        await test.assert.didSignal(trigger, 'TP.sig.UIDidActivate');

        systemPopup = TP.byId('systemPopup', windowContext);
        test.assert.isVisible(systemPopup);

        test.getSuite().resetSignalTracking();

        await driver.constructSequence().
                        keyUp('[Enter]', trigger).
                        run();

        test.refute.hasAttribute(trigger, 'pclass:active');

        await test.assert.didSignal(trigger, 'TP.sig.UIDeactivate');
        await test.assert.didSignal(trigger, 'TP.sig.UIDidDeactivate');

        systemPopup = TP.byId('systemPopup', windowContext);
        test.assert.isVisible(systemPopup);

        test.getSuite().resetSignalTracking();

        await driver.constructSequence().
                        keyDown('[Enter]', trigger).
                        run();

        test.assert.hasAttribute(trigger, 'pclass:active');

        await test.assert.didSignal(trigger, 'TP.sig.UIActivate');
        await test.assert.didSignal(trigger, 'TP.sig.UIDidActivate');

        systemPopup = TP.byId('systemPopup', windowContext);
        test.assert.isVisible(systemPopup);

        test.getSuite().resetSignalTracking();

        await driver.constructSequence().
                        keyUp('[Enter]', trigger).
                        run();

        test.refute.hasAttribute(trigger, 'pclass:active');

        await test.assert.didSignal(trigger, 'TP.sig.UIDeactivate');
        await test.assert.didSignal(trigger, 'TP.sig.UIDidDeactivate');

        systemPopup = TP.byId('systemPopup', windowContext);
        test.refute.isVisible(systemPopup);
    });

    //  ---

    this.it('Disabled behavior', async function(test, options) {

        var trigger,
            systemPopup;

        //  All of these mechanisms to 'activate' the control should fail - it's
        //  disabled.

        //  Disable it
        trigger = TP.byId('trigger4', windowContext);
        await trigger.setAttrDisabled(true);

        //  --- Focus

        await driver.constructSequence().
                        sendEvent(TP.hc('type', 'focus'), trigger).
                        run();

        test.refute.hasAttribute(trigger, 'pclass:focus');

        await test.refute.didSignal(trigger, 'TP.sig.UIFocus');
        await test.refute.didSignal(trigger, 'TP.sig.UIDidFocus');

        test.getSuite().resetSignalTracking();

        //  --- Individual mousedown/mouseup don't affect popups

        //  --- click

        await driver.constructSequence().
                        click(TP.LEFT, trigger).
                        run();

        await test.refute.didSignal(trigger, 'TP.sig.UIActivate');
        await test.refute.didSignal(trigger, 'TP.sig.UIDidActivate');

        await test.refute.didSignal(trigger, 'TP.sig.UIDeactivate');
        await test.refute.didSignal(trigger, 'TP.sig.UIDidDeactivate');

        systemPopup = TP.byId('systemPopup', windowContext);
        test.refute.isVisible(systemPopup);

        test.getSuite().resetSignalTracking();

        //  --- Individual keydown/keyup

        await driver.constructSequence().
                    keyDown('[Enter]', trigger).
                    run();

        test.refute.hasAttribute(trigger, 'pclass:active');

        await test.refute.didSignal(trigger, 'TP.sig.UIActivate');
        await test.refute.didSignal(trigger, 'TP.sig.UIDidActivate');

        systemPopup = TP.byId('systemPopup', windowContext);
        test.refute.isVisible(systemPopup);

        test.getSuite().resetSignalTracking();

        await driver.constructSequence().
                    keyUp('[Enter]', trigger).
                    run();

        await test.refute.didSignal(trigger, 'TP.sig.UIDeactivate');
        await test.refute.didSignal(trigger, 'TP.sig.UIDidDeactivate');

        systemPopup = TP.byId('systemPopup', windowContext);
        test.refute.isVisible(systemPopup);
    });
});

//  ------------------------------------------------------------------------
//  end
//  ========================================================================
